var http = require('http')
  , https = require('https')
  , _ = require('underscore');

function Mailer(config, host) {
  // Force new instance.
  if ( ! (this instanceof Mailer)) {
    return new Mailer(config);
  }
  // Allow string configs.
  if (typeof config !== 'object') { config = { key: config }; }
  if (typeof host === 'string') { config.host = host; }

  // Merge with sensible defaults.
  this.config = { ssl: true };
  _.defaults(this.config, config);
}

// Send the message.
Mailer.prototype.send = function(callback) {
  // Make callback optional.
  if (typeof callback !== 'function') {
    callback = function(){};
  }
  
  // Prepare post data.
  var post_data = [
    this.config.from
    , this.config.to
    , ""
    , "From: "+this.config.from
    , "To: "+this.config.to
    , "Content-Type: text/plain;charset=utf-8"
    , "Subject: "+this.config.subject
    , ""
    , this.config.body
  ].join("\n");

  // Prepare request options.
  var options = {
    host: 'mailgun.net'
    , port: this.config.ssl ? 443 : 80
    , path: '/api/messages.eml?servername='+this.config.host
    , method: 'POST'
    , headers: {
      'Authorization': 'Basic '+(new Buffer('api:'+this.config.key).toString("base64"))
      , 'Content-Type': 'text/plain'
      , 'Content-Length': post_data.length
    }
  };

  // Prepare request object.
  (this.config.ssl ? https : http).request(options, function(res) {
    callback(res.statusCode !== 201 ? res.statusCode : null);
  
  // Handle errors.
  }).on('error', function(err) {
    callback(err.message);
  
  // Send request.
  }).end(post_data);
};

// Add message configs.
Mailer.prototype.add = function(key, val) {
  if (typeof key === 'object') {
    _.defaults(this.config, key);
  } else {
    this.config[key] = val;
  }
  return this;
};

// Remove message configs.
Mailer.prototype.remove = function() {
  var args = Array.prototype.slice.call(arguments);
  args.forEach(function(key){
    delete this.config[key];
  });
  return this;
}

module.exports = Mailer;